


global ro_L ro_G g Beta Uf Ul UG Rf Ub Rs d Ut

d=0.385; %inner diameter
Uls=2; %liquid superficial
Ugs=2;%gas superficial
DX=0.01; %space step
angles=30; %inclination angle
Lu=80*d; %unit length


xxx=0:DX:Lu;
ii=0;
for Beta=angles*pi/180
  
%%%Input values

A=(pi*d^2)/4; %cross area
ro_L=790; %API 48 crude oil density kg/m^3 
nu_L=3.8*10^(-6); %liquid kinematic viscosity m^2/s (correction reference paper)
ro_G=0.675; %methane density kg/m^3 
nu_G=1.615*10^(-5); %gas kinematic viscosity m^2/s (correction reference paper))
g=9.81; %gravitational acceleration
SIG=0.001; %surface tension PARAMETER
Sigma=SIG*g*(ro_L-ro_G)*(d^2)/4; %surface tension


%%%physical values for in liquid slug
Us=Uls+Ugs; %mixture velocity
B=1+0.2*sin(Beta)^2; %distribution parameter
U0=1.54*(Sigma*g*(ro_L-ro_G)/(ro_L)^2)^(1/4); %bubble rise velocity m/s
Ub=B*Us+U0; %bubble velocity in liquid slug m/s
%Ud=0.4*sqrt(g*d); %bubble drift velocity of large bubble m/s
Ud=0.54*sqrt(g*d)*cos(Beta)+0.35*sqrt(g*d)*sin(Beta);

C=1.2; %factor dependent on surface tension and Re number

%C=2; %factor dependent on surface tension and Re number

Ut=C*Us+Ud; %translational velocity of slug unit
Rs=1/(1+(Us/8.66)^(1.39)); %liquid hold-up in liquid slug
Alp_s=1-Rs; %gas fraction in liquid slug
Ul=(Us-Ub*Alp_s)/Rs; %Liquid velocity in the liquid slug




%%%initial hf value hs
x0 =d/2; 
fun = @sol_hfis;
[x,ff,exitflag] = fsolve(fun,x0,optimset('display','off')); % Call solver

if exitflag~=1
    error('Error. \invalid hfi');
end

conv=fun(x);
hfi=x;
hf=hfi;
HFI(ii+1)=hfi;
Rf=Rs; %initial value for Rf
Uf=Ul; %initial value for Uf
UG=Ub; %initial value for UG


i=1;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                            Integration Start
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Res=1;
dz=DX; %space increment
z=0;
zz(1)=0;
hff(1)=hf;
Uff(1)=Uf;
UGG(1)=UG;
Rff(1)=Rf;

q=2*acos(1-2*hf/d);
Sf=q*d/2;
SG=pi*d-Sf;
Si=d/2*sqrt(2-2*cos(q));
Af=((d/2)^2)/2*(q-sin(q));
AG=((pi*d^2)/4)-Af;
Uf=Ut-(Ut-Ul)*Rs/Rf; %liquid velocity in flim
Alp_f=Alp_s;
UG=Ut-(Ut-Ub)*Alp_s/Alp_f; %gas velocity in film
Dhf=4*Af/Sf; %liquid hydraulic diameter
DhG=4*AG/(SG+Si); %liquid hydraulic diameter

Re_fs=abs(d*Ul/(nu_L)); %Re number for liquid in slug

Re_f=abs(Dhf*Uf/(nu_L)); %Re number for liquid
Re_G=abs(DhG*UG/(nu_G));  %Re number for gas

if abs(Re_f)<2300  %critical value between laminar vs turbulent flow
Ck=16; %pamameters for friction coefficients 
n=-1;  %pamameters for friction coefficients 
else
Ck=0.046; %pamameters for friction coefficients 
n=-0.2;  %pamameters for friction coefficients 
end

if abs(Re_fs)<2300  %critical value between laminar vs turbulent flow
Ck_s=16; %pamameters for friction coefficients 
n_s=-1;  %pamameters for friction coefficients 
else
Ck_s=0.046; %pamameters for friction coefficients 
n_s=-0.2;  %pamameters for friction coefficients 
end

ffs=Ck_s*Re_fs^(n_s); %friction coef for liquid-wall
tau_s=ffs*(ro_L*abs(Ul)*Ul/2);
Fr_slug(1)=(tau_s*pi*d/A);

 %critical value hfc

   x00 =d/2; 
   fun2 = @sol_hfc;
   options = optimoptions('fsolve','Display','iter'); % Option to display output
   [hc,resi,exitflag2] = fsolve(fun2,x00,optimset('display','off')); % Call solver
   
     if exitflag2==1 || exitflag2==-2
        if hc*0.99<hfi
           hf=hc*0.99;
        end
    end




while Res>0



for j=1:4
   



Alp_f=1-Rf;




%%%geometrical relationship
q=2*acos(1-2*hf(j)/d);
Sf=q*d/2;
SG=pi*d-Sf;
Si=d/2*sqrt(2-2*cos(q));
Af=((d/2)^2)/2*(q-sin(q));
AG=((pi*d^2)/4)-Af;


%%%UG & Uf
Uf=Ut-(Ut-Ul)*Rs/Rf; %liquid velocity in flim
UG=Ut-(Ut-Ub)*Alp_s/Alp_f; %gas velocity in film


%%%forces
%turbulent flow 
% Ck=0.046;
% n=-0.2;
% Laminar



Dhf=4*Af/Sf; %liquid hydraulic diameter
DhG=4*AG/(SG+Si); %liquid hydraulic diameter

Re_fs=abs(d*Ul/(nu_L)); %Re number for liquid in slug

Re_f=abs(Dhf*Uf/(nu_L)); %Re number for liquid
Re_G=abs(DhG*UG/(nu_G));  %Re number for gas

if abs(Re_f)<2300  %critical value between laminar vs turbulent flow
Ck=16; %pamameters for friction coefficients 
n=-1;  %pamameters for friction coefficients 
else
Ck=0.046; %pamameters for friction coefficients 
n=-0.2;  %pamameters for friction coefficients 
end

if abs(Re_fs)<2300 %critical value between laminar vs turbulent flow
Ck_s=16; %pamameters for friction coefficients 
n_s=-1;  %pamameters for friction coefficients 
else
Ck_s=0.046; %pamameters for friction coefficients 
n_s=-0.2;  %pamameters for friction coefficients 
end

ff=Ck*Re_f^(n); %friction coef for liquid-wall
ffs=Ck_s*Re_fs^(n_s); %friction coef for liquid-wall
fG=Ck*Re_G^(n); %friction coef for gas-wall
fi=0.014; %interficial coef for gas-liquid

tau_f=ff*(ro_L*abs(Uf)*Uf/2);
tau_G=fG*(ro_G*abs(UG)*UG/2);
tau_i=fi*(ro_G*abs(UG-Uf)*(UG-Uf)/2);

tau_s=ffs*(ro_L*abs(Ul)*Ul/2);

dRDh=(4/(pi*d)*sqrt(1-(2*hf(j)/d-1)^2));
fenzi=tau_f*Sf/Af-tau_G*SG/AG-tau_i*Si*(1/Af+1/AG)+(ro_L-ro_G)*g*sin(Beta);
fenmu=((ro_L-ro_G)*g*cos(Beta)-ro_L*(Ut-Uf)*(Ut-Ul)*Rs/(Rf^2)*dRDh-ro_G*(Ut-UG)*(Ut-Ub)*((1-Rs)/(1-Rf)^2)*dRDh);

K(j)=fenzi/fenmu; %The film equation in Taitel 
if j<3
hf(j+1)=hf(1)+K(j)*dz/2;
else
hf(j+1)=hf(1)+K(j)*dz;   
end
Rf=1/pi*(pi-acos(2*hf(j+1)/d-1)+(2*hf(j+1)/d-1)*sqrt(1-(2*hf(j+1)/d-1)^2));

end

K_ave=1/6*(K(1)+2*K(2)+2*K(3)+K(4));
hff(i+1)=hf(1)+K_ave*dz;

Rff(i+1)=1/pi*(pi-acos(2*hff(i+1)/d-1)+(2*hff(i+1)/d-1)*sqrt(1-(2*hff(i+1)/d-1)^2));
Uff(i+1)=Ut-(Ut-Ul)*Rs/Rff(i+1); %liquid velocity in flim
UGG(i+1)=Ut-(Ut-Ub)*Alp_s/(1-Rff(i+1)); %gas velocity in film


hf=hff(i+1);
Rf=1/pi*(pi-acos(2*hf/d-1)+(2*hf/d-1)*sqrt(1-(2*hf/d-1)^2));



%%%cross-sectional geometrical relationship
q=2*acos(1-2*hff(i+1)/d);
Sf=q*d/2;
SG=pi*d-Sf;
Si=d/2*sqrt(2-2*cos(q));
Af=((d/2)^2)/2*(q-sin(q));
AG=((pi*d^2)/4)-Af;
Dhf=4*Af/Sf; %liquid hydraulic diameter

Re_f=Dhf*Uff(i+1)/(nu_L); %Re number for liquid
Re_G=DhG*UGG(i+1)/(nu_G);  %Re number for gas
if Re_f<2300 %critical value between laminar vs turbulent flow
Ck=16; %pamameters for friction coefficients 
n=-1;  %pamameters for friction coefficients 
else
Ck=0.046; %pamameters for friction coefficients 
n=-0.2;  %pamameters for friction coefficients 
end

ff=Ck*Re_f^(n); %friction coef for liquid-wall
fG=Ck*Re_G^(n); %friction coef for gas-wall

tau_ff(i+1)=ff*(ro_L*abs(Uff(i+1))*Uff(i+1)/2);
tau_GG(i+1)=fG*(ro_G*abs(UGG(i+1))*UGG(i+1)/2);

Sff(i+1)=Sf;
SGG(i+1)=SG;


z=z+dz;
zz(i+1)=z;


Convv(i)=Ul*Rs+Ut*(1-Rs)*z/Lu-(Ut/Lu)*(trapz(zz,1-Rff))-Uls;
Res=Convv(i);



%pressure drop
Alp_u=(Ugs-Ub*Alp_s+Ut*Alp_s)/Ut;
ro_u=Alp_u*ro_G+(1-Alp_u)*ro_L;
Fr_slug(i+1)=(tau_ff(i+1).*Sff(i+1)+tau_GG(i+1).*SGG(i+1))./A;



i=i+1;
end

Pzz=0:DX:Lu;
if z>dz
    if Lu-z<=0
        PD_dynamic=(trapz(Pzz,Fr_slug(1:length(Pzz))));  %we assume that when Lu<z, the pressure loss from liquid slug is ignored 
        PD_static=ro_u*g*sin(Beta)*Lu;
    else
PD_dynamic=tau_s*pi*d/A*(Lu-z)+(trapz(zz,Fr_slug));
PD_static=ro_u*g*sin(Beta)*Lu;
    end
end




N=Lu/DX+1;
N2=round(N);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%SLUG UNIT BUILD UP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 FILM=fliplr(hff);
 SLUG=ones(1,round((Lu-z)/dz))*d;
if z>=Lu
 SLUGUNIT2=real(FILM);
else
 SLUGUNIT2=[SLUG,real(FILM)];
end
 SLUGUNIT=SLUGUNIT2(1:N2);
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%HOLD-UP BUILD UP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 R_FILM=fliplr(Rff);
 R_SLUG=ones(1,round((Lu-z)/dz))*Rs;
if z>=Lu
R_SLUGUNIT2=R_FILM;
else
R_SLUGUNIT2=[R_SLUG,R_FILM];
end
R_SLUGUNIT=R_SLUGUNIT2(1:N2);
% 
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%LIQUID BUILD UP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 LV_FILM=[fliplr(Uff)];
 LV_SLUG=ones(1,round((Lu-z)/dz))*Ul;
if z>=Lu
 LV_SLUGUNIT2=LV_FILM;
else
LV_SLUGUNIT2=[LV_SLUG,LV_FILM];
end
 LV_SLUGUNIT=LV_SLUGUNIT2(1:N2);
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%GAS BUILD UP
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
GV_FILM=[fliplr(UGG)];
GV_SLUG=ones(1,round((Lu-z)/dz))*Ub;
if z>=Lu
GV_SLUGUNIT2=GV_FILM;
else
GV_SLUGUNIT2=[GV_SLUG,GV_FILM];
end
GV_SLUGUNIT=GV_SLUGUNIT2(1:N2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %%%pressure calculation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


P_FILM=[fliplr(Fr_slug)];
P_SLUG=ones(1,round((Lu-z)/dz))*(tau_s*pi*d/A);
if z>=Lu
P_DY2=P_FILM;
else
P_DY2=[P_SLUG,P_FILM];
end
P_DYUNIT=P_DY2(1:N2);
P_STAUNIT=(R_SLUGUNIT*ro_L+(1-R_SLUGUNIT)*ro_G)*g*sin(Beta);
P_TOUNIT=P_DYUNIT+P_STAUNIT;

PPD_dy(ii+1)=PD_dynamic;
PPD_sta(ii+1)=PD_static;


if ii==0
SLUGMO((1:length(SLUGUNIT))+length(SLUGUNIT)*ii)=SLUGUNIT; %film height
R_SLUGMO((1:length(R_SLUGUNIT))+(length(R_SLUGUNIT)*ii))=R_SLUGUNIT;  %liquid hold-up
LV_SLUGMO((1:length(LV_SLUGUNIT))+length(LV_SLUGUNIT)*ii)=LV_SLUGUNIT; %liquid velocity 
GV_SLUGMO((1:length(GV_SLUGUNIT))+length(GV_SLUGUNIT)*ii)=GV_SLUGUNIT; %gas velocity 
P_SLUGMO((1:length(P_TOUNIT))+length(P_TOUNIT)*ii)=P_TOUNIT; %pressure distribution 
else 
SLUGMO((1:length(SLUGUNIT))+((length(SLUGUNIT)-1)*ii))=SLUGUNIT; %film height
R_SLUGMO((1:length(R_SLUGUNIT))+((length(R_SLUGUNIT)-1)*ii))=R_SLUGUNIT;  %liquid hold-up
LV_SLUGMO((1:length(LV_SLUGUNIT))+((length(LV_SLUGUNIT)-1)*ii))=LV_SLUGUNIT; %liquid velocity 
GV_SLUGMO((1:length(GV_SLUGUNIT))+((length(GV_SLUGUNIT)-1)*ii))=GV_SLUGUNIT; %gas velocity 
P_SLUGMO((1:length(P_TOUNIT))+((length(P_TOUNIT)-1)*ii))=P_TOUNIT; %pressure distribution 
end



 



% %%%%five points


ii=ii+1;

hff=0;
Rff=0;
Uff=0;
UGG=0;
Convv=0;
zz=0;
tau_ff=0;
tau_GG=0;
Sff=0;
SGG=0;
Fr_slug=0;
end

 zzz=(0:length(SLUGMO)-1)*dz;

display('slug model ok');



figure(1)
hold on
plot(xxx,SLUGMO)
xlabel('x')
ylabel('height(m)')

figure(2)
hold on
plot(xxx,R_SLUGMO)
xlabel('x')
ylabel('R(%)')
% % 
figure(3)
hold on
plot(xxx,LV_SLUGMO,'b-')
xlabel('x')
ylabel('Lv(m/s)')

figure(4)
hold on
plot(xxx,GV_SLUGMO,'k-')
xlabel('x')
ylabel('Gv(m/s)')











